<p class="logo">A <a href="http://pocoo.org/">
  <img src="{{ pathto("_static/pocoo.png", 1) }}" alt="Pocoo" /></a>
  {%trans%}project{%endtrans%}</p>

<h3>Download</h3>
{% if version.endswith('a0') %}
<p>{%trans%}This documentation is for version <b>{{ version }}</b>, which is
  not released yet.{%endtrans%}</p>
<p>{%trans%}You can use it from the
  <a href="https://github.com/sphinx-doc/sphinx/">Git repo</a> or look for
  released versions in the <a href="https://pypi.python.org/pypi/Sphinx">Python
    Package Index</a>.{%endtrans%}</p>
{% else %}
<p>{%trans%}Current version: <b>{{ version }}</b>{%endtrans%}</p>
<p>{%trans%}Get Sphinx from the <a href="https://pypi.python.org/pypi/Sphinx">Python Package
Index</a>, or install it with:{%endtrans%}</p>
<pre>pip install -U Sphinx</pre>
{% endif %}

<h3>{%trans%}Questions? Suggestions?{%endtrans%}</h3>

<p>{%trans%}Join the <a href="http://groups.google.com/group/sphinx-users">sphinx-users</a> mailing list on Google Groups:{%endtrans%}</p>
<form action="http://groups.google.com/group/sphinx-users/boxsubscribe"
      style="padding-left: 0.5em">
  <input type="text" name="email" value="your@email" style="font-size: 90%; width: 120px"
         onfocus="$(this).val('');"/>
  <input type="submit" name="sub" value="Subscribe" style="font-size: 90%; width: 70px"/>
</form>
<p>{%trans%}or come to the <tt>#sphinx-doc</tt> channel on FreeNode.{%endtrans%}</p>
<p>{%trans%}You can also open an issue at the
  <a href="https://github.com/sphinx-doc/sphinx/issues">tracker</a>.{%endtrans%}</p>
