Documentation.addTranslations({"locale": "lt", "plural_expr": "(n%10==1 && n%100!=11 ? 0 : n%10>=2 && (n%100<10 || n%100>=20) ? 1 : 2)", "messages": {"Next topic": "Kita tema", "Index": "Indeksas", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "", "Copyright": "Autoriaus teis\u0117s", "C API changes": "C API pakeitimai", "quick access to all modules": "greitas vis\u0173 moduli\u0173 pasiekimas", "&copy; Copyright %(copyright)s.": "&copy; Autoriaus teis\u0117s %(copyright)s.", "Global Module Index": "Globalus Modulio Indeksas", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "Index &ndash; %(key)s": "Indeksas &ndash; %(key)s", "General Index": "Bendras indeksas", "next chapter": "kita dalis", "Search finished, found %s page(s) matching the search query.": "", "previous chapter": "praeita dalis", "Permalink to this headline": "Nuoroda \u012f \u0161i\u0105 antra\u0161t\u0119", "About these documents": "Apie \u0161iuos dokumentus", "Preparing search...": "", ", in ": "", "Navigation": "Navigacija", "Expand sidebar": "I\u0161pl\u0117sti \u0161onin\u0119 juost\u0105", "the documentation for": "", "Complete Table of Contents": "Pilnas Turinys", "Contents": "Turinys", "can be huge": "gali b\u016bti didelis", "Changes in Version %(version)s &mdash; %(docstitle)s": "Pasikeitimai versijoje %(version)s &mdash; %(docstitle)s", "Other changes": "Kiti pakeitimai", "Hide Search Matches": "Pasl\u0117pti paie\u0161kos rezultatus", "Quick search": "Greitoji paie\u0161ka", "Show Source": "Rodyti pirmin\u012f kod\u0105", "Search": "Paie\u0161ka", "This Page": "\u0160is puslapis", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "\u010cia j\u016bs galite ie\u0161koti \u0161iuose dokumentuose. \u012eveskite savo paie\u0161kos\n    \u017eod\u017eius \u012f lauk\u0105 apa\u010dioje ir paspauskite \"ie\u0161koti\". Pasteb\u0117sime, kad paie\u0161kos\n    funkcija automati\u0161kai ie\u0161kos vis\u0173 \u017eod\u017ei\u0173. Puslapiai,\n    kuriuose yra ma\u017eiau \u017eod\u017ei\u0173 nepasirodys tarp paie\u0161kos rezultat\u0173.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Sukurta naudojant <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "", "Collapse sidebar": "Pasl\u0117pti \u0161onin\u0119 juost\u0105", "Go": "Pirmyn", "Table Of Contents": "Turinys", "Search within %(docstitle)s": "Ie\u0161koti tarp %(docstitle)s", "all functions, classes, terms": "visos funkcijos, klas\u0117s ir terminai", "Please activate JavaScript to enable the search\n    functionality.": "Pra\u0161ome aktyvuoti JavaScript, kad veikt\u0173 paie\u0161kos\n    funkcionalumas.", "Indices and tables:": "Indeksai ir lentel\u0117s:", "lists all sections and subsections": "sura\u0161yti visus skyrius ir poskyrius", "Index pages by letter": "Indekso puslapiai pagal raid\u0119", "search": "ie\u0161koti", "Permalink to this definition": "Nuoroda \u012f \u0161\u012f apibr\u0117\u017eim\u0105", "Previous topic": "Praeita tema", "Overview": "Ap\u017evalga", "Last updated on %(last_updated)s.": "Paskutinis atnaujinimas %(last_updated)s.", "Searching": "", "search this documentation": "ie\u0161koti \u0161iame dokumente", "Automatically generated list of changes in version %(version)s": "Automati\u0161kai sugeneruotas pakeitim\u0173 %(version)s versijoje s\u0105ra\u0161as", "Full index on one page": "Pilnas indeksas viename puslapyje", "Enter search terms or a module, class or function name.": "\u012eveskite paie\u0161kos termin\u0105 arba modulio, klas\u0117s ar funkcijos vard\u0105.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Autoriaus teis\u0117s</a> %(copyright)s.", "Library changes": "Bibliotekos pakeitimai", "Search Page": "Paie\u0161kos puslapis", "Search Results": "Paie\u0161kos rezultatai"}});