Documentation.addTranslations({"locale": "ko", "plural_expr": "0", "messages": {"Next topic": "\ub2e4\uc74c \ud56d\ubaa9", "Index": "\uc0c9\uc778", "%(filename)s &mdash; %(docstitle)s": "", "Welcome! This is": "", "Copyright": "\uc800\uc791\uad8c", "C API changes": "C API\uc5d0 \ub300\ud55c \ubcc0\uacbd", "quick access to all modules": "\ubaa8\ub4e0 \ubaa8\ub4c8 \uc870\uacac\ud45c", "&copy; Copyright %(copyright)s.": "", "Global Module Index": "\ubaa8\ub4c8 \ucd1d \uc0c9\uc778", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "Index &ndash; %(key)s": "", "General Index": "\uc804\uccb4 \uc0c9\uc778", "next chapter": "\ub2e4\uc74c \uc7a5", "Search finished, found %s page(s) matching the search query.": "", "previous chapter": "\uc774\uc804 \uc7a5", "Permalink to this headline": "\uc81c\ubaa9 \uc8fc\uc18c", "About these documents": "\uc774 \ubb38\uc11c \uc815\ubcf4", "Preparing search...": "", ", in ": "", "Navigation": "\ud0d0\uc0c9", "Expand sidebar": "\uc0ac\uc774\ub4dc\ubc14 \uc5f4\uae30", "the documentation for": "", "Complete Table of Contents": "\uc885\ud569 \ubaa9\ucc28", "Contents": "\ub0b4\uc6a9", "can be huge": "\ud070 \uacbd\uc6b0\uac00 \uc788\uc73c\ubbc0\ub85c \uc8fc\uc758", "Changes in Version %(version)s &mdash; %(docstitle)s": "", "Other changes": "\ub2e4\ub978 \ubcc0\uacbd \uc0ac\ud56d", "Hide Search Matches": "\uac80\uc0c9 \uacb0\uacfc \uc228\uae30\uae30", "Quick search": "\ube60\ub978 \uac80\uc0c9", "Show Source": "\uc18c\uc2a4 \ucf54\ub4dc\ub97c \ubcf4\ub824\uba74", "Search": "\uac80\uc0c9", "This Page": "\ud604\uc7ac \ubb38\uc11c", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "", "last updated": "", "Collapse sidebar": "\uc0ac\uc774\ub4dc\ubc14 \ub2eb\uae30", "Go": "\ubc14\ub85c \uac00\uae30", "Table Of Contents": "\ubaa9\ucc28", "Search within %(docstitle)s": "%(docstitle)s\uc5d0\uc11c \ucc3e\uae30", "all functions, classes, terms": "\ud568\uc218, \ud074\ub798\uc2a4 \ubc0f \uc6a9\uc5b4 \uac1c\uad00", "Please activate JavaScript to enable the search\n    functionality.": "", "Indices and tables:": "\uc0c9\uc778 \ubc0f \ud45c \ubaa9\ub85d:", "lists all sections and subsections": "\uc601\uc5ed\ubcc4 \ubaa9\ucc28", "Index pages by letter": "\uc54c\ud30c\ubcb3\ubcc4 \uc0c9\uc778", "search": "\uac80\uc0c9", "Permalink to this definition": "\uc815\uc758 \uc8fc\uc18c", "Previous topic": "\uc774\uc804 \ud56d\ubaa9", "Overview": "\uac1c\uc694", "Last updated on %(last_updated)s.": "\ucd5c\uc885 \uc5c5\ub370\uc774\ud2b8: %(last_updated)s", "Searching": "", "search this documentation": "\ubb38\uc11c \uac80\uc0c9", "Automatically generated list of changes in version %(version)s": "\ubc84\uc804 %(version)s\uc758 \ubcc0\uacbd \uc0ac\ud56d (\uc774 \ubaa9\ub85d\uc740 \uc790\ub3d9\uc73c\ub85c \uc0dd\uc131\ud569\ub2c8\ub2e4)", "Full index on one page": "\uc77c\ubc18 \uc0c9\uc778", "Enter search terms or a module, class or function name.": "\ubaa8\ub4c8, \ud074\ub798\uc2a4 \ub610\ub294 \ud568\uc218 \uc774\ub984\uc744 \uc785\ub825\ud558\uc2ed\uc2dc\uc624.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "", "Library changes": "\ub77c\uc774\ube0c\ub7ec\ub9ac\uc5d0 \ub300\ud55c \ubcc0\uacbd", "Search Page": "\uac80\uc0c9 \ud398\uc774\uc9c0", "Search Results": "\uac80\uc0c9 \uacb0\uacfc"}});