Documentation.addTranslations({"locale": "hu", "plural_expr": "(n != 1)", "messages": {"Next topic": "K\u00f6vetkez\u0151 t\u00e9mak\u00f6r", "Index": "T\u00e1rgymutat\u00f3", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "\u00dcdv\u00f6z\u00f6lj\u00fck! Ez a", "Copyright": "Minden jog fenntartva", "C API changes": "C API v\u00e1ltoz\u00e1sok", "quick access to all modules": "gyors hozz\u00e1f\u00e9r\u00e9s az \u00f6sszes modulhoz", "&copy; Copyright %(copyright)s.": "&copy; Minden jog fenntartva %(copyright)s.", "Global Module Index": "Teljes modul t\u00e1rgymutat\u00f3", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "A keres\u00e9se nem hozott eredm\u00e9nyt. Ellen\u0151rizze, a megadott kulcsszavakat \u00e9s azt, hogy megfelel\u0151 sz\u00e1m\u00fa kateg\u00f3ria van-e kiv\u00e1lasztva.", "Index &ndash; %(key)s": "T\u00e1rgymutat\u00f3 &ndash; %(key)s", "General Index": "\u00c1ltal\u00e1nos t\u00e1rgymutat\u00f3", "next chapter": "k\u00f6vetkez\u0151 fejezet", "Search finished, found %s page(s) matching the search query.": "A keres\u00e9s befejez\u0151d\u00f6tt, %s oldal egyezik a keres\u00e9si fel\u00e9teleknek.", "previous chapter": "el\u0151z\u0151 fejezet", "Permalink to this headline": "Hivatkoz\u00e1s erre a fejezetc\u00edmre", "About these documents": "N\u00e9vjegy ezekr\u0151l a dokumentumokr\u00f3l", "Preparing search...": "Felk\u00e9sz\u00fcl\u00e9s a keres\u00e9sre...", ", in ": ", ", "Navigation": "Navig\u00e1ci\u00f3", "Expand sidebar": "Oldals\u00e1v kinyit\u00e1sa", "the documentation for": "dokument\u00e1ci\u00f3", "Complete Table of Contents": "Teljes tartalomjegyz\u00e9k", "Contents": "Tartalom", "can be huge": "nagy lehet", "Changes in Version %(version)s &mdash; %(docstitle)s": "V\u00e1ltoz\u00e1sok a(z) %(version)s v\u00e1ltozatban &mdash; %(docstitle)s", "Other changes": "Egy\u00e9b v\u00e1ltoz\u00e1sok", "Hide Search Matches": "Keres\u00e9si Tal\u00e1latok Elrejt\u00e9se", "Quick search": "Gyorskeres\u00e9s", "Show Source": "Forr\u00e1s megtekint\u00e9se", "Search": "Keres\u00e9s", "This Page": "Ez az Oldal", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Err\u0151l az oldalr\u00f3l ind\u00edthatja keres\u00e9seit. \u00cdrja be a kulcsszavakat\n    az al\u00e1bbi sz\u00f6vegdobozba, majd kattintson a \"keres\u00e9s\" gombra.\n    \u00dcgyeljen arra, hogy a keres\u00e9s megadott kulcsszavak mindegyik\u00e9t\n    figyelembe veszi, \u00edgy azok az oldalak, melyek nem tartalmazz\u00e1k az\n    \u00f6sszes kifejez\u00e9st, nem jelennek meg a tal\u00e1lati list\u00e1ban.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "<a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s haszn\u00e1lat\u00e1val k\u00e9sz\u00fclt.", "last updated": "utolj\u00e1ra friss\u00edtve", "Collapse sidebar": "Oldals\u00e1v \u00f6sszez\u00e1r\u00e1sa", "Go": "Ok", "Table Of Contents": "Tartalomjegyz\u00e9k", "Search within %(docstitle)s": "Keres\u00e9s k\u00f6zt\u00fck: %(docstitle)s", "all functions, classes, terms": "\u00f6sszes funkci\u00f3, oszt\u00e1ly \u00e9s kifejez\u00e9s", "Please activate JavaScript to enable the search\n    functionality.": "K\u00e9rem enged\u00e9lyezze a JavaScriptet a keres\u0151 funkci\u00f3\n    haszn\u00e1lat\u00e1hoz.", "Indices and tables:": "T\u00e1rgymutat\u00f3 \u00e9s t\u00e1bl\u00e1zatok", "lists all sections and subsections": "kilist\u00e1zza az \u00f6sszes fejezetet \u00e9s alfejezetet", "Index pages by letter": "Oldalak ABC sorrendben", "search": "keres\u00e9s", "Permalink to this definition": "Hivatkoz\u00e1s erre a defin\u00edci\u00f3ra", "Previous topic": "El\u0151z\u0151 t\u00e9mak\u00f6r", "Overview": "\u00c1ttekint\u00e9s", "Last updated on %(last_updated)s.": "Utols\u00f3 friss\u00edt\u00e9s %(last_updated)s.", "Searching": "Keres\u00e9s folyamatban", "search this documentation": "keres\u00e9s ebben a dokument\u00e1ci\u00f3ban", "Automatically generated list of changes in version %(version)s": "Automatikusan gener\u00e1lt v\u00e1ltoz\u00e1slista a(z) %(version)s v\u00e1ltozathoz", "Full index on one page": "Teljes t\u00e1rgymutat\u00f3 egy oldalon", "Enter search terms or a module, class or function name.": "Adjon meg egy keresend\u0151 kifejez\u00e9st, modul, oszt\u00e1ly vagy funkci\u00f3 nevet.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Minden jog fenntartva</a> %(copyright)s.", "Library changes": "K\u00f6nyvt\u00e1r v\u00e1ltoz\u00e1sok", "Search Page": "Keres\u00e9s", "Search Results": "Keres\u00e9si Eredm\u00e9nyek"}});