Documentation.addTranslations({"locale": "et", "plural_expr": "(n != 1)", "messages": {"Next topic": "J\u00e4rgmine teema", "Index": "Indeks", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "Tervitused! See on", "Copyright": "Autori\u00f5igus", "C API changes": "C API muutused", "quick access to all modules": "kiire ligip\u00e4\u00e4s k\u00f5igile moodulitele", "&copy; Copyright %(copyright)s.": "&copy; Autori\u00f5igused %(copyright)s.", "Global Module Index": "Globaalne moodulite indeks", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "Sinu otsingule ei vastanud \u00fckski dokument. Palun veendu, et k\u00f5ik sisestatud s\u00f5nad on \u00f5igesti kirjutatud ja sa oled valikud piisaval hulgal kategooriaid.", "Index &ndash; %(key)s": "Indeks &ndash; %(key)s", "General Index": "\u00dcldindeks", "next chapter": "j\u00e4rgmine jaotis", "Search finished, found %s page(s) matching the search query.": "Otsingu tulemusena leiti %s leht(e).", "previous chapter": "eelmine jaotis", "Permalink to this headline": "P\u00fcsiviit sellele pealkirjale", "About these documents": "Info selle dokumentatsiooni kohta", "Preparing search...": "Otsingu ettevalmistamine...", ", in ": "", "Navigation": "Navigatsioon", "Expand sidebar": "N\u00e4ita k\u00fclgriba", "the documentation for": "dokumentatsioon projektile", "Complete Table of Contents": "T\u00e4ielik sisukord", "Contents": "Sisukord", "can be huge": "v\u00f5ib olla v\u00e4ga suur", "Changes in Version %(version)s &mdash; %(docstitle)s": "Muutused versioonis %(version)s &mdash; %(docstitle)s", "Other changes": "\u00dclej\u00e4\u00e4nud muutused", "Hide Search Matches": "Varja otsingu tulemused", "Quick search": "Kiirotsing", "Show Source": "N\u00e4ita l\u00e4htekoodi", "Search": "Otsing", "This Page": "K\u00e4esolev leht", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Selle vormi abil saab otsida k\u00e4esolevast dokumentatsioonist. Sisesta allolevasse lahtrisse otsis\u00f5nad ning kl\u00f5psa \"Otsi\". Tasub t\u00e4hele panna, et otsingu tulemuses kuvatakse k\u00f5iki otsis\u00f5nasid sisaldavaid lehti.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Loodud <a href=\"http://sphinx-doc.org/\">Sphinxiga</a> (versioon: %(sphinx_version)s).", "last updated": "viimati uuendatud", "Collapse sidebar": "Varja k\u00fclgriba", "Go": "Otsi", "Table Of Contents": "Sisukord", "Search within %(docstitle)s": "Otsi %(docstitle)s piires", "all functions, classes, terms": "k\u00f5ik funktsioonid, klassid ja terminid", "Please activate JavaScript to enable the search\n    functionality.": "Otsingu v\u00f5imaldamiseks tuleb aktiveerida JavaScript.", "Indices and tables:": "Indeksid ja tabelid", "lists all sections and subsections": "toob v\u00e4lja k\u00f5ik sektsioonid ja alamsektsioonid", "Index pages by letter": "Indeksi lehek\u00fcljed algust\u00e4he kaupa", "search": "otsi", "Permalink to this definition": "P\u00fcsiviit sellele definitsioonile", "Previous topic": "Eelmine teema", "Overview": "\u00dclevaade", "Last updated on %(last_updated)s.": "Viimati uuendatud %(last_updated)s.", "Searching": "Otsimine", "search this documentation": "otsi sellest dokumentatsioonist", "Automatically generated list of changes in version %(version)s": "Automaatselt genereeritud nimekiri versiooni %(version)s muutustest", "Full index on one page": "T\u00e4isindeks \u00fchel lehel", "Enter search terms or a module, class or function name.": "Sisesta otsingus\u00f5na v\u00f5i mooduli/klassi/funktsiooni nimi.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Autori\u00f5igus</a> %(copyright)s.", "Library changes": "Teegi muutused", "Search Page": "Otsinguleht", "Search Results": "Otsingu tulemused"}});