Documentation.addTranslations({"locale": "id", "plural_expr": "0", "messages": {"Next topic": "Topik berikutnya", "Index": "Indeks", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "Selamat Datang! Ini adalah", "Copyright": "Copyright", "C API changes": "Perubahan API C", "quick access to all modules": "akses cepat semua modul", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "Index Modul Global", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "Tidak ada dokumen yang cocok dengan pencarian anda. Pastikan semua kata ditulis dengan benar dan sudah memilih cukup kategori.", "Index &ndash; %(key)s": "Index &ndash; %(key)s", "General Index": "Indeks Umum", "next chapter": "bab berikutnya", "Search finished, found %s page(s) matching the search query.": "Pencarian selesai, menemukan %s halaman yang cocok dengan kueri pencarian.", "previous chapter": "bab sebelum", "Permalink to this headline": "Link permanen untuk headline ini", "About these documents": "Tentang dokumen ini", "Preparing search...": "Penyiapkan pencarian...", ", in ": ", di", "Navigation": "Navigasi", "Expand sidebar": "Buka sidebar", "the documentation for": "dokumentasi untuk", "Complete Table of Contents": "Daftar Isi Lengkap", "Contents": "Konten", "can be huge": "dapat menjadi besar", "Changes in Version %(version)s &mdash; %(docstitle)s": "Perubahan pada Versi %(version)s &mdash; %(docstitle)s", "Other changes": "Perubahan lain", "Hide Search Matches": "Sembunyikan Hasil Pencarian", "Quick search": "Pencarian cepat", "Show Source": "Lihat Sumber", "Search": "Pencarian", "This Page": "Halaman Ini", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Dari sini dapat dilakukan pencarian pada dokumentasi. Masukkan\n    kata yang dicari pada kotak dibawah dan klik \"search\". Catatan untuk fungsi pencarian\n    akan secara otomatis mencari semua kata. Halaman\n    yang berisi kata yang sedikat tidak dimunculkan pada daftar hasil.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Dibuat menggunakan <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "terakhir diperbarui", "Collapse sidebar": "Tutup sidebar", "Go": "Go", "Table Of Contents": "Daftar Isi", "Search within %(docstitle)s": "Pencarian dalam %(docstitle)s", "all functions, classes, terms": "semua fungsi, class, term", "Please activate JavaScript to enable the search\n    functionality.": "Tolong aktifkan JavaScript untuk melakukan pencarian.\n ", "Indices and tables:": "Index dan tabel:", "lists all sections and subsections": "daftar semua seksi dan subseksi", "Index pages by letter": "Index halaman berdasarkan huruf", "search": "pencarian", "Permalink to this definition": "Link permanen untuk definisi ini", "Previous topic": "Topik sebelum", "Overview": "Tinjauan", "Last updated on %(last_updated)s.": "Terakhir diperbarui pada %(last_updated)s.", "Searching": "Pencarian", "search this documentation": "pencarian pada dokumentasi ini", "Automatically generated list of changes in version %(version)s": "Daftar perubahan dibuat otomatis untuk versi %(version)s", "Full index on one page": "Index penuh dalam satu halaman", "Enter search terms or a module, class or function name.": "Masukkan term pencarian atau nama modul, class atau fungsi.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.", "Library changes": "Perubahan library", "Search Page": "Pencarian Halaman", "Search Results": "Hasil Pencarian"}});