Documentation.addTranslations({"locale": "pt_PT", "plural_expr": "(n != 1)", "messages": {"Next topic": "Pr\u00f3ximo t\u00f3pico", "Index": "\u00cdndice", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "Bem Vindo(a)! Esta \u00e9", "Copyright": "Copyright", "C API changes": "Altera\u00e7\u00f5es na API C", "quick access to all modules": "acesso r\u00e1pido a todos os m\u00f3dulos", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "\u00cdndice Global de M\u00f3dulos", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "A sua pesquisa n\u00e3o encontrou nenhum documento. Por favor, confirme que todas as palavras est\u00e3o escritas corretamente e que selecionou categorias suficientes.", "Index &ndash; %(key)s": "\u00cdndice &ndash; %(key)s", "General Index": "\u00cdndice Geral", "next chapter": "pr\u00f3ximo cap\u00edtulo", "Search finished, found %s page(s) matching the search query.": "Pesquisa conclu\u00edda, foram encontrada(s) %s p\u00e1gina(s) que combinam com a consulta feita.", "previous chapter": "cap\u00edtulo anterior", "Permalink to this headline": "Link permanente para este t\u00edtulo", "About these documents": "Sobre estes documentos", "Preparing search...": "A preparar a pesquisa...", ", in ": ", em", "Navigation": "Navega\u00e7\u00e3o", "Expand sidebar": "Expandir painel lateral", "the documentation for": "a documenta\u00e7\u00e3o de", "Complete Table of Contents": "Tabela de Conte\u00fados Completa", "Contents": "Conte\u00fado", "can be huge": "pode ser enorme", "Changes in Version %(version)s &mdash; %(docstitle)s": "Altera\u00e7\u00f5es na Vers\u00e3o%(version)s &mdash; %(docstitle)s", "Other changes": "Outras altera\u00e7\u00f5es", "Hide Search Matches": "Esconder Resultados da Pesquisa", "Quick search": "Pesquisa r\u00e1pida", "Show Source": "Exibir Fonte", "Search": "Pesquisar", "This Page": "Esta P\u00e1gina", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "A partir daqui pode pesquisar estes documentos. Preencha as\npalavras de pesquisa na caixa abaixo e clique em  \"pesquisar\".\nNote que a fun\u00e7\u00e3o de pesquisa ir\u00e1 procurar automaticamente\npor todas as palavras. P\u00e1ginas que contenham menos palavras\nn\u00e3o ir\u00e3o aparecer na lista de resultados.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Criado utilizando <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "\u00faltima actualiza\u00e7\u00e3o", "Collapse sidebar": "Recolher painel lateral", "Go": "Ir", "Table Of Contents": "Tabela de Conte\u00fados", "Search within %(docstitle)s": "Pesquisar dentro de %(docstitle)s", "all functions, classes, terms": "todas as fun\u00e7\u00f5es, classes, termos", "Please activate JavaScript to enable the search\n    functionality.": "Por favor ligue o JavaScript para habilitar a\nfuncionalidade de pesquisa.", "Indices and tables:": "\u00cdndices e tabelas:", "lists all sections and subsections": "Listar todas as sec\u00e7\u00f5es e subsec\u00e7\u00f5es", "Index pages by letter": "Paginas de \u00edndice por letra", "search": "pesquisar", "Permalink to this definition": "Link permanente para esta defini\u00e7\u00e3o", "Previous topic": "T\u00f3pico anterior", "Overview": "Vis\u00e3o geral", "Last updated on %(last_updated)s.": "\u00daltima actualiza\u00e7\u00e3o em %(last_updated)s.", "Searching": "A Pesquisar", "search this documentation": "Pesquisar esta documenta\u00e7\u00e3o", "Automatically generated list of changes in version %(version)s": "Lista de altera\u00e7\u00f5es gerada automaticamente na vers\u00e3o %(version)s", "Full index on one page": "\u00cdndice completo numa p\u00e1gina", "Enter search terms or a module, class or function name.": "Digite os termos da busca ou o nome de um m\u00f3dulo, classe ou fun\u00e7\u00e3o.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.", "Library changes": "Altera\u00e7\u00f5es na biblioteca", "Search Page": "P\u00e1gina de Pesquisa", "Search Results": "Resultados da Pesquisa"}});