Documentation.addTranslations({"locale": "tr", "plural_expr": "(n > 1)", "messages": {"Next topic": "Sonraki konu", "Index": "Dizin", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "Ho\u015fgeldiniz! Kar\u015f\u0131n\u0131zda", "Copyright": "Copyright", "C API changes": "C API'sindeki de\u011fi\u015fiklikler", "quick access to all modules": "b\u00fct\u00fcn mod\u00fcllere h\u0131zl\u0131 eri\u015fim", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "Global Mod\u00fcl Dizini", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "Arama sonucunda herhangi bir belge bulunamad\u0131. B\u00fct\u00fcn kelimeleri do\u011fru yazd\u0131\u011f\u0131n\u0131zdan ve gerekli b\u00fct\u00fcn kategorileri se\u00e7ti\u011finizden emin olun.", "Index &ndash; %(key)s": "Dizin &ndash; %(key)s", "General Index": "Genel Dizin", "next chapter": "sonraki b\u00f6l\u00fcm", "Search finished, found %s page(s) matching the search query.": "Arama tamamland\u0131. Sorguyu i\u00e7eren %s sayfa bulundu.", "previous chapter": "\u00f6nceki b\u00f6l\u00fcm", "Permalink to this headline": "Bu ba\u015fl\u0131\u011f\u0131n kal\u0131c\u0131 ba\u011flant\u0131s\u0131", "About these documents": "Bu belgeler hakk\u0131nda", "Preparing search...": "Aramaya haz\u0131rlan\u0131yor...", ", in ": ", \u015funun i\u00e7inde:", "Navigation": "Gezinti", "Expand sidebar": "Yan \u00e7ubu\u011fu geni\u015flet", "the documentation for": "belgelendirme konusu: ", "Complete Table of Contents": "Ayr\u0131nt\u0131l\u0131 \u0130\u00e7indekiler Tablosu", "Contents": "\u0130\u00e7indekiler", "can be huge": "\u00e7ok b\u00fcy\u00fck olabilir", "Changes in Version %(version)s &mdash; %(docstitle)s": "S\u00fcr\u00fcm %(version)s &mdash; %(docstitle)s i\u00e7indeki De\u011fi\u015fiklikler", "Other changes": "Di\u011fer de\u011fi\u015fiklikler", "Hide Search Matches": "Arama Sonu\u00e7lar\u0131n\u0131 Gizle", "Quick search": "H\u0131zl\u0131 Arama", "Show Source": "Kayna\u011f\u0131 G\u00f6ster", "Search": "Ara", "This Page": "Bu Sayfa", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Burada belgeler i\u00e7inde arama yapabilirsiniz. Arad\u0131\u011f\u0131n\u0131z kelimeyi \na\u015fa\u011f\u0131daki kutuya yaz\u0131p \"ara\" d\u00fc\u011fmesine bas\u0131n\u0131z. Arama i\u015flevi \notomatik olarak b\u00fct\u00fcn kelimeleri arayacakt\u0131r. Eksik kelime i\u00e7eren \nsayfalar sonu\u00e7 listesinde g\u00f6r\u00fcnmez.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "<a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s ile olu\u015fturulmu\u015ftur.", "last updated": "son g\u00fcncelleme", "Collapse sidebar": "Yan \u00e7ubu\u011fu daralt", "Go": "Git", "Table Of Contents": "\u0130\u00e7indekiler Tablosu", "Search within %(docstitle)s": "%(docstitle)s i\u00e7inde ara", "all functions, classes, terms": "b\u00fct\u00fcn fonksiyonlar, s\u0131n\u0131flar, terimler", "Please activate JavaScript to enable the search\n    functionality.": "Arama i\u015flevini kullanabilmek i\u00e7in l\u00fctfen JavaScript'i\n    etkinle\u015ftirin.", "Indices and tables:": "Dizinler ve tablolar", "lists all sections and subsections": "b\u00fct\u00fcn b\u00f6l\u00fcmler ve alt b\u00f6l\u00fcmler listelenir", "Index pages by letter": "Harfe g\u00f6re dizin sayfalar\u0131", "search": "ara", "Permalink to this definition": "Bu tan\u0131m\u0131n kal\u0131c\u0131 ba\u011flant\u0131s\u0131", "Previous topic": "\u00d6nceki konu", "Overview": "Genel Bak\u0131\u015f", "Last updated on %(last_updated)s.": "Son g\u00fcncelleme: %(last_updated)s.", "Searching": "Aran\u0131yor", "search this documentation": "Bu belgelerde ara", "Automatically generated list of changes in version %(version)s": "%(version)s s\u00fcr\u00fcm\u00fcndeki de\u011fi\u015fikliklerin otomatik olarak \u00fcretilmi\u015f listesi", "Full index on one page": "B\u00fct\u00fcn dizin tek sayfada", "Enter search terms or a module, class or function name.": "Aranacak terimleri veya mod\u00fcl, s\u0131n\u0131f ya da fonksiyon ad\u0131n\u0131 yaz\u0131n\u0131z", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.", "Library changes": "K\u00fct\u00fcphane de\u011fi\u015fiklikleri", "Search Page": "Arama Sayfas\u0131", "Search Results": "Arama Sonu\u00e7lar\u0131"}});