Documentation.addTranslations({"locale": "hr", "plural_expr": "n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2", "messages": {"Next topic": "Slijede\u0107a tema", "Index": "Abecedni popis", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "", "Copyright": "Sva prava zadr\u017eana", "C API changes": "C API changes", "quick access to all modules": "brz dostup do svih modulov", "&copy; Copyright %(copyright)s.": "&copy; Sva prava zadr\u017eana %(copyright)s.", "Global Module Index": "Op\u0107eniti popis modula", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "Index &ndash; %(key)s": "Index &ndash; %(key)s", "General Index": "Opceniti abecedni indeks", "next chapter": "slijede\u0107e poglavje", "Search finished, found %s page(s) matching the search query.": "", "previous chapter": "Prija\u0161nje poglavje", "Permalink to this headline": "Link na taj naslov", "About these documents": "O ovim dokumentima", "Preparing search...": "", ", in ": "", "Navigation": "Navigacija", "Expand sidebar": "", "the documentation for": "", "Complete Table of Contents": "Potpuna tabela sadr\u017eaja", "Contents": "", "can be huge": "mo\u017ee biti veliko", "Changes in Version %(version)s &mdash; %(docstitle)s": "Changes in Version %(version)s &mdash; %(docstitle)s", "Other changes": "Ostale promjene", "Hide Search Matches": "Sakrij rezultate pretrage", "Quick search": "Brzo pretra\u017eivanje", "Show Source": "Prika\u017ei izvorni kod", "Search": "Tra\u017ei", "This Page": "Trenutna stranica", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Izra\u0111eno sa <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "", "Collapse sidebar": "", "Go": "Naprijed", "Table Of Contents": "Pregled sadr\u017eaja", "Search within %(docstitle)s": "Tra\u017ei izme\u0111u %(docstitle)s", "all functions, classes, terms": "sve funkcije, razredi, izrazi", "Please activate JavaScript to enable the search\n    functionality.": "Molimo omogu\u0107ite JavaScript\n    za djelovanje tra\u017eilice.", "Indices and tables:": "Kazala i tabele:", "lists all sections and subsections": "prika\u017ei sve sekcije i podsekcije", "Index pages by letter": "Indeksiraj stranice po slovu", "search": "tra\u017ei", "Permalink to this definition": "Link na tu definiciju", "Previous topic": "Prija\u0161nja tema", "Overview": "Pregled", "Last updated on %(last_updated)s.": "Zadnji put a\u017eurirano %(last_updated)s.", "Searching": "", "search this documentation": "tra\u017ei po dokumentaciji", "Automatically generated list of changes in version %(version)s": "Automatically generated list of changes in version %(version)s", "Full index on one page": "Potpun indeks na jednoj strani", "Enter search terms or a module, class or function name.": "Unesi ime modula, razreda ili funkcije.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Sva prava zadr\u017eana</a> %(copyright)s.", "Library changes": "Library changes", "Search Page": "Tra\u017eilica", "Search Results": "Rezultati pretrage"}});