Documentation.addTranslations({"locale": "nl", "plural_expr": "(n != 1)", "messages": {"Next topic": "Volgend onderwerp", "Index": "Index", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "", "Copyright": "Copyright", "C API changes": "Veranderingen in de C-API", "quick access to all modules": "sneltoegang naar alle modules", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "Globale Module-index", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "Index &ndash; %(key)s": "Index &ndash; %(key)s", "General Index": "Algemene index", "next chapter": "volgend hoofdstuk", "Search finished, found %s page(s) matching the search query.": "", "previous chapter": "Vorig hoofdstuk", "Permalink to this headline": "Permalink naar deze titel", "About these documents": "Over deze documenten", "Preparing search...": "", ", in ": "", "Navigation": "Navigatie", "Expand sidebar": "", "the documentation for": "", "Complete Table of Contents": "Volledige inhoudsopgave", "Contents": "Inhoud", "can be huge": "kan heel groot zijn", "Changes in Version %(version)s &mdash; %(docstitle)s": "Veranderingen in versie %(version)s &mdash; %(docstitle)s", "Other changes": "Andere veranderingen", "Hide Search Matches": "Zoekresultaten verbergen", "Quick search": "Snel zoeken", "Show Source": "Broncode weergeven", "Search": "Zoeken", "This Page": "Deze pagina", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Hier kan u de documenten doorzoeken.  Geef enkele trefwoorden\n    in het veld hieronder en klik \"zoeken\".  Merk op dat de zoekfunctie\n    steeds naar alle woorden zoekt.  Pagina's die minder woorden bevatten\n    zullen niet tussen de resultaten verschijnen.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Aangemaakt met <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "", "Collapse sidebar": "", "Go": "Ga", "Table Of Contents": "Inhoudsopgave", "Search within %(docstitle)s": "Zoeken in %(docstitle)s", "all functions, classes, terms": "alle functies, klasses en begrippen", "Please activate JavaScript to enable the search\n    functionality.": "Activeer JavaSscript om de zoekfunctionaliteit in te schakelen.", "Indices and tables:": "Indices en tabellen:", "lists all sections and subsections": "geeft alle secties en subsecties weer", "Index pages by letter": "Index pagineerd per letter", "search": "zoeken", "Permalink to this definition": "Permalink naar deze definitie", "Previous topic": "Vorig onderwerp", "Overview": "Overzicht", "Last updated on %(last_updated)s.": "Laatste aanpassing op %(last_updated)s.", "Searching": "", "search this documentation": "zoeken in deze documentatie", "Automatically generated list of changes in version %(version)s": "Automatisch gegenereerde lijst van veranderingen in versie %(version)s", "Full index on one page": "Volledige index op een pagina", "Enter search terms or a module, class or function name.": "Geef zoekterm of de naam van een module, klasse of functie.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.", "Library changes": "Veranderingen in de bibliotheek", "Search Page": "Zoekpagina", "Search Results": "Zoekresultaten"}});