Documentation.addTranslations({"locale": "fr", "plural_expr": "(n > 1)", "messages": {"Next topic": "Sujet suivant", "Index": "Index", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "Bienvenue ! Ceci est", "Copyright": "Copyright", "C API changes": "Modifications de l'API C", "quick access to all modules": "acc\u00e8s rapide \u00e0 l'ensemble des modules", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "Index g\u00e9n\u00e9ral des modules", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "Votre recherche ne correspond \u00e0 aucun document. Veuillez v\u00e9rifier que les mots sont correctement orthographi\u00e9s et que vous avez s\u00e9lectionn\u00e9 assez de cat\u00e9gories.", "Index &ndash; %(key)s": "Index &ndash; %(key)s", "General Index": "Index g\u00e9n\u00e9ral", "next chapter": "Chapitre suivant", "Search finished, found %s page(s) matching the search query.": "La recherche est finie, %s page(s) trouv\u00e9e(s) qui corresponde(nt) \u00e0 la recherche.", "previous chapter": "Chapitre pr\u00e9c\u00e9dent", "Permalink to this headline": "Lien permanent vers ce titre", "About these documents": "\u00c0 propos de ces documents", "Preparing search...": "Pr\u00e9paration de la recherche...", ", in ": ", dans", "Navigation": "Navigation", "Expand sidebar": "Agrandir la barre lat\u00e9rale", "the documentation for": "la documentation pour", "Complete Table of Contents": "Table des mati\u00e8res compl\u00e8te", "Contents": "Contenu", "can be huge": "peut \u00eatre \u00e9norme", "Changes in Version %(version)s &mdash; %(docstitle)s": "Modifications dans la version %(version)s &mdash; %(docstitle)s", "Other changes": "Autres modifications", "Hide Search Matches": "Cacher les r\u00e9sultats de la recherche", "Quick search": "Recherche rapide", "Show Source": "Montrer le code source", "Search": "Recherche", "This Page": "Cette page", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Vous pouvez effectuer une recherche au sein des documents. Saisissez les termes\nde votre recherche dans le champs ci-dessous et cliquez sur \"rechercher\". Notez que la fonctionnalit\u00e9 de recherche\nva automatiquement chercher l'ensemble des mots. Les pages\ncontenant moins de mots n'appara\u00eetront pas dans la liste des r\u00e9sultats.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Cr\u00e9\u00e9 avec <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "derni\u00e8re modification", "Collapse sidebar": "R\u00e9duire la barre lat\u00e9rale", "Go": "Go", "Table Of Contents": "Table des Mati\u00e8res", "Search within %(docstitle)s": "Recherchez dans %(docstitle)s", "all functions, classes, terms": "toutes les fonctions, classes, termes", "Please activate JavaScript to enable the search\n    functionality.": "Veuillez activer le JavaScript pour que la recherche fonctionne.", "Indices and tables:": "Indices et Tables :", "lists all sections and subsections": "lister l'ensemble des sections et sous-sections", "Index pages by letter": "Indexer les pages par lettre", "search": "rechercher", "Permalink to this definition": "Lien permanent vers cette d\u00e9finition", "Previous topic": "Sujet pr\u00e9c\u00e9dent", "Overview": "R\u00e9sum\u00e9", "Last updated on %(last_updated)s.": "Mis \u00e0 jour le %(last_updated)s.", "Searching": "Recherche en cours", "search this documentation": "rechercher dans cette documentation", "Automatically generated list of changes in version %(version)s": "Liste auto-g\u00e9n\u00e9r\u00e9e des modifications dans la version %(version)s", "Full index on one page": "Index complet sur une seule page", "Enter search terms or a module, class or function name.": "Saisissez un mot clef ou un nom de module, classe ou fonction.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.", "Library changes": "Modifications de la biblioth\u00e8que", "Search Page": "Page de recherche", "Search Results": "R\u00e9sultats de la recherche"}});