Documentation.addTranslations({"locale": "ca", "plural_expr": "(n != 1)", "messages": {"Next topic": "Tema seg\u00fcent", "Index": "\u00cdndex", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "", "Copyright": "Copyright", "C API changes": "Canvis a la API de C", "quick access to all modules": "acc\u00e9s r\u00e0pid a tots els m\u00f2duls", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "\u00cdndex Global de M\u00f2duls", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "Index &ndash; %(key)s": "\u00cdndes &ndash; %(key)s", "General Index": "\u00cdndex General", "next chapter": "cap\u00edtol seg\u00fcent", "Search finished, found %s page(s) matching the search query.": "", "previous chapter": "cap\u00edtol anterior", "Permalink to this headline": "Link permanent a aquest t\u00edtol", "About these documents": "Quant a aquests documents", "Preparing search...": "", ", in ": "", "Navigation": "Navegaci\u00f3", "Expand sidebar": "", "the documentation for": "", "Complete Table of Contents": "Taula de Contingut Completa", "Contents": "", "can be huge": "pot ser gegant", "Changes in Version %(version)s &mdash; %(docstitle)s": "Canvis a la Versi\u00f3 %(version)s &mdash; %(docstitle)s", "Other changes": "Altres canvis", "Hide Search Matches": "Oculta Resultats de Cerca", "Quick search": "Cerca r\u00e0pida", "Show Source": "Mostra Codi Font", "Search": "Cerca", "This Page": "Aquesta P\u00e0gina", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Des d'aqu\u00ed pots fer cerques en aquests documents. Entra les \nparaules de la teva cerca i clica el bot\u00f3 \"cerca\". Tingues en compte\nque la cerca inclour\u00e0 totes les paraules que posis. Les p\u00e0gines que no\ntenen totes les paraules no sortir\u00e0n.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Creat amb <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "", "Collapse sidebar": "", "Go": "Ves a", "Table Of Contents": "Taula de Contingut", "Search within %(docstitle)s": "Cerca dins de %(docstitle)s", "all functions, classes, terms": "totes les funcions, classes, termes", "Please activate JavaScript to enable the search\n    functionality.": "Activa JavaScript per utilitzar la funcionalitat\nde cerca.", "Indices and tables:": "\u00cdndexs i taules:", "lists all sections and subsections": "llista totes les seccions i subseccions", "Index pages by letter": "P\u00e0gines d'\u00edndex per lletra", "search": "cerca", "Permalink to this definition": "Link permanent a aquesta definici\u00f3", "Previous topic": "Tema anterior", "Overview": "Resum", "Last updated on %(last_updated)s.": "\u00daltima actualitzaci\u00f3 el %(last_updated)s.", "Searching": "", "search this documentation": "cerca aquesta documentaci\u00f3", "Automatically generated list of changes in version %(version)s": "Llista de canvis de la versi\u00f3 %(version)s generada autom\u00e0ticament", "Full index on one page": "\u00cdndex complet en una p\u00e0gina", "Enter search terms or a module, class or function name.": "Entra paraules de cerca o el nom d'un m\u00f2dul, classe o funci\u00f3.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\\\"%(path)s\\\">Copyright</a> %(copyright)s.", "Library changes": "Canvis a la llibreria", "Search Page": "P\u00e0gina de Cerca", "Search Results": "Resultats de la Cerca"}});