Documentation.addTranslations({"locale": "it", "plural_expr": "(n != 1)", "messages": {"Next topic": "Argomento successivo", "Index": "Indice", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "Benvenuto! Questa \u00e8", "Copyright": "Copyright", "C API changes": "Modifiche nelle API C", "quick access to all modules": "accesso veloce ai moduli", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "Indice dei moduli", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "La tua ricerca non corrisponde a nessun documento. Verifica che tutte le parole siano scritte correttamente e di aver scelto un numero sufficiente di categorie.", "Index &ndash; %(key)s": "Indice &ndash; %(key)s", "General Index": "Indice generale", "next chapter": "capitolo successivo", "Search finished, found %s page(s) matching the search query.": "Ricerca completata, trovata/e %s pagina/e corrispondenti.", "previous chapter": "capitolo precedente", "Permalink to this headline": "Link a questa intestazione", "About these documents": "A proposito di questi documenti", "Preparing search...": "Preparo la ricerca...", ", in ": ", in", "Navigation": "Navigazione", "Expand sidebar": "Espandi la barra laterale", "the documentation for": "la documentazione per", "Complete Table of Contents": "Tabella dei contenuti completa", "Contents": "Contenuti", "can be huge": "pu\u00f2 essere enorme", "Changes in Version %(version)s &mdash; %(docstitle)s": "Modifiche nella  Versione %(version)s &mdash; %(docstitle)s", "Other changes": "Altre modifiche", "Hide Search Matches": "Nascondi i risultati della ricerca", "Quick search": "Ricerca veloce", "Show Source": "Mostra sorgente", "Search": "Cerca", "This Page": "Questa pagina", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Puoi effettuare una ricerca in questi documenti. Immetti le parole chiave \n    della tua ricerca nel riquadro sottostante \"cerca\". Nota che la funzione\n    di ricerca cerca automaticamente per tutte le parole. Le pagine\n    che contendono meno parole non compariranno nei risultati di ricerca.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Creato con <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "ultimo aggiornamento", "Collapse sidebar": "Comprimi la barra laterale", "Go": "Vai", "Table Of Contents": "Tabella dei contenuti", "Search within %(docstitle)s": "Cerca in %(docstitle)s", "all functions, classes, terms": "tutte le funzioni, classi e moduli", "Please activate JavaScript to enable the search\n    functionality.": "Attiva JavaScript per abilitare la funzione\u23ce\ndi ricerca.", "Indices and tables:": "Indici e tabelle:", "lists all sections and subsections": "elenca l'insieme delle sezioni e sottosezioni", "Index pages by letter": "Indice delle pagine per lettera", "search": "cerca", "Permalink to this definition": "Link a questa definizione", "Previous topic": "Argomento precedente", "Overview": "Sintesi", "Last updated on %(last_updated)s.": "Ultimo aggiornamento %(last_updated)s.", "Searching": "Cerca", "search this documentation": "cerca in questa documentazione", "Automatically generated list of changes in version %(version)s": "Lista delle modifiche generata automaticamente nella versione %(version)s", "Full index on one page": "Indice completo in una pagina", "Enter search terms or a module, class or function name.": "Inserisci un termine di ricerca un modulo, classe o nome di funzione", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.", "Library changes": "Modifiche nella libreria", "Search Page": "Cerca", "Search Results": "Risultati della ricerca"}});