Documentation.addTranslations({"locale": "es", "plural_expr": "(n != 1)", "messages": {"Next topic": "Pr\u00f3ximo tema", "Index": "\u00cdndice", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "\u00a1Bienvenido! Este es", "Copyright": "Copyright", "C API changes": "Cambios en la API C", "quick access to all modules": "acceso r\u00e1pido a todos los m\u00f3dulos", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "\u00cdndice Global de M\u00f3dulos", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "Su b\u00fasqueda no coincide con ning\u00fan documentos. Por favor, aseg\u00farese de que todas las palabras est\u00e9n correctamente escritas y que usted all\u00e1 seleccionado las suficientes categor\u00edas.", "Index &ndash; %(key)s": "\u00cdndice &ndash; %(key)s", "General Index": "\u00cdndice General", "next chapter": "pr\u00f3ximo cap\u00edtulo", "Search finished, found %s page(s) matching the search query.": "B\u00fasqueda finalizada, encontr\u00f3 %s p\u00e1gina(s) acorde con la consulta de b\u00fasqueda.", "previous chapter": "cap\u00edtulo anterior", "Permalink to this headline": "Enlazar permanentemente con este t\u00edtulo", "About these documents": "Sobre este documento", "Preparing search...": "Preparando b\u00fasqueda...", ", in ": ", en ", "Navigation": "Navegaci\u00f3n", "Expand sidebar": "Expandir barra lateral", "the documentation for": "la documentaci\u00f3n para", "Complete Table of Contents": "\u00cdndice de contenidos completo", "Contents": "Contenidos", "can be huge": "puede ser muy grande", "Changes in Version %(version)s &mdash; %(docstitle)s": "Cambios en la versi\u00f3n %(version)s &mdash; %(docstitle)s", "Other changes": "Otros cambios", "Hide Search Matches": "Ocultar coincidencias de la b\u00fasqueda", "Quick search": "B\u00fasqueda r\u00e1pida", "Show Source": "Mostrar el c\u00f3digo", "Search": "B\u00fasqueda", "This Page": "Esta p\u00e1gina", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Este es el di\u00e1logo de b\u00fasqueda.  Introduce los t\u00e9rminos en el\n    di\u00e1logo siguiente y pulsa \"buscar\". Note que el asistente buscar\u00e1 \n    autom\u00e1ticamente todas las palabras. Las p\u00e1ginas que contengan \n    menos palabras no aparecer\u00e1n en la lista de resultados.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Creado con <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "actualizado por \u00faltima vez el", "Collapse sidebar": "Contraer barra lateral", "Go": "Ir a", "Table Of Contents": "Tabla de Contenidos", "Search within %(docstitle)s": "Buscar en %(docstitle)s", "all functions, classes, terms": "todas las funciones, clases, t\u00e9rminos", "Please activate JavaScript to enable the search\n    functionality.": "Por favor, active JavaScript para habilitar la funcionalidad\n    de b\u00fasqueda.", "Indices and tables:": "\u00cdndices y tablas:", "lists all sections and subsections": "muestra todas las secciones y subsecciones", "Index pages by letter": "\u00cdndice alfab\u00e9tico de p\u00e1ginas", "search": "buscar", "Permalink to this definition": "Enlazar permanentemente con esta definici\u00f3n", "Previous topic": "Tema anterior", "Overview": "Resumen", "Last updated on %(last_updated)s.": "Actualizado por \u00faltima vez en %(last_updated)s.", "Searching": "Buscando", "search this documentation": "buscar en esta documentaci\u00f3n", "Automatically generated list of changes in version %(version)s": "Lista de cambios generada autom\u00e1ticamente en la versi\u00f3n %(version)s", "Full index on one page": "\u00cdndice completo en una p\u00e1gina", "Enter search terms or a module, class or function name.": "Introduzca los t\u00e9rminos de b\u00fasqueda o un nombre de m\u00f3dulo, clase o funci\u00f3n.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\\\"%(path)s\\\">Copyright</a> %(copyright)s.", "Library changes": "Cambios en la biblioteca", "Search Page": "P\u00e1gina de B\u00fasqueda", "Search Results": "Resultados de la b\u00fasqueda"}});