Documentation.addTranslations({"locale": "de", "plural_expr": "(n != 1)", "messages": {"Next topic": "N\u00e4chstes Thema", "Index": "Stichwortverzeichnis", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "Willkommen! Dies ist", "Copyright": "Copyright", "C API changes": "C API-\u00c4nderungen", "quick access to all modules": "schneller Zugriff auf alle Module", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "Globaler Modulindex", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "Ihre Suche ergab keine Treffer. Bitte stellen Sie sicher, dass alle W\u00f6rter richtig geschrieben sind und gen\u00fcgend Kategorien ausgew\u00e4hlt sind.", "Index &ndash; %(key)s": "Stichwortverzeichnis &ndash; %(key)s", "General Index": "Stichwortverzeichnis", "next chapter": "n\u00e4chstes Kapitel", "Search finished, found %s page(s) matching the search query.": "Die Suche ist fertig, es wurde(n) %s Seite(n) mit Treffern gefunden.", "previous chapter": "vorheriges Kapitel", "Permalink to this headline": "Link zu dieser \u00dcberschrift", "About these documents": "\u00dcber dieses Dokument", "Preparing search...": "Suche wird vorbereitet...", ", in ": ", in ", "Navigation": "Navigation", "Expand sidebar": "Seitenleiste ausklappen", "the documentation for": "die Dokumentation f\u00fcr", "Complete Table of Contents": "Vollst\u00e4ndiges Inhaltsverzeichnis", "Contents": "Inhalt", "can be huge": "kann gro\u00df sein", "Changes in Version %(version)s &mdash; %(docstitle)s": "\u00c4nderungen in Version %(version)s &mdash; %(docstitle)s", "Other changes": "Andere \u00c4nderungen", "Hide Search Matches": "Suchergebnisse ausblenden", "Quick search": "Schnellsuche", "Show Source": "Quellcode anzeigen", "Search": "Suche", "This Page": "Diese Seite", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Von hier aus k\u00f6nnen Sie die Dokumentation durchsuchen. Geben Sie Ihre Suchbegriffe in das untenstehende Feld ein und klicken Sie auf \"Suchen\". Bitte beachten Sie, dass die Suchfunktion automatisch nach allen Worten sucht. Seiten, die nicht alle Worte enthalten, erscheinen nicht in der Ergebnisliste.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Mit <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s erstellt.", "last updated": "zuletzt aktualisiert", "Collapse sidebar": "Seitenleiste einklappen", "Go": "Los", "Table Of Contents": "Inhalt", "Search within %(docstitle)s": "Suche in %(docstitle)s", "all functions, classes, terms": "alle Funktionen, Klassen, Begriffe", "Please activate JavaScript to enable the search\n    functionality.": "Bitte aktivieren Sie JavaScript, wenn Sie die Suchfunktion nutzen wollen.", "Indices and tables:": "Verzeichnisse und Tabellen:", "lists all sections and subsections": "Liste aller Kapitel und Unterkapitel", "Index pages by letter": "Stichwortverzeichnis nach Anfangsbuchstabe", "search": "suchen", "Permalink to this definition": "Link zu dieser Definition", "Previous topic": "Vorheriges Thema", "Overview": "\u00dcbersicht", "Last updated on %(last_updated)s.": "Zuletzt aktualisiert am %(last_updated)s.", "Searching": "Suchen", "search this documentation": "durchsuche diese Dokumentation", "Automatically generated list of changes in version %(version)s": "Automatisch generierte Liste der \u00c4nderungen in Version %(version)s", "Full index on one page": "Gesamtes Stichwortverzeichnis auf einer Seite", "Enter search terms or a module, class or function name.": "Geben Sie Suchbegriffe oder einen Modul-, Klassen- oder Funktionsnamen ein.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.", "Library changes": "Bibliotheks-\u00c4nderungen", "Search Page": "Suche", "Search Results": "Suchergebnisse"}});