Documentation.addTranslations({"locale": "nb_NO", "plural_expr": "(n != 1)", "messages": {"Next topic": "Neste emne", "Index": "Index", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "", "Copyright": "Copyright", "C API changes": "Endringer i C API", "quick access to all modules": "snarvei til alle moduler", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "Global Modulindex", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "Index &ndash; %(key)s": "Index &ndash; %(key)s", "General Index": "Hovedindex", "next chapter": "neste kapittel", "Search finished, found %s page(s) matching the search query.": "", "previous chapter": "Forrige kapittel", "Permalink to this headline": "Permalink til denne oversikten", "About these documents": "Om disse dokumenter", "Preparing search...": "", ", in ": "", "Navigation": "Navigering", "Expand sidebar": "Utvid sidepanelet", "the documentation for": "", "Complete Table of Contents": "Komplett Innholdsfortegnelse", "Contents": "Innhold", "can be huge": "kan bli stor", "Changes in Version %(version)s &mdash; %(docstitle)s": "Endringer i version %(version)s &mdash; %(docstitle)s", "Other changes": "Andre endringer", "Hide Search Matches": "Skjul s\u00f8keresultat", "Quick search": "Hurtigs\u00f8k", "Show Source": "Vis kildekode", "Search": "S\u00f8k", "This Page": "Denne siden", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "her kan du s\u00f8ke blant disse dokumentene. Angi s\u00f8keord nedfor og klikk \"s\u00f8k\".\n    S\u00f8ket m\u00e5 treffe p\u00e5 samtlige s\u00f8keord.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Lagd med <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "", "Collapse sidebar": "Skjul sidepanelet", "Go": "G\u00e5", "Table Of Contents": "Innholdsfortegnelse", "Search within %(docstitle)s": "S\u00f8k blant %(docstitle)s", "all functions, classes, terms": "alla funksjoner, klasser, vilk\u00e5r", "Please activate JavaScript to enable the search\n    functionality.": "Vennligst aktiver JavaScript for \u00e5 aktivere s\u00f8k.", "Indices and tables:": "Index og tabeller", "lists all sections and subsections": "liste over alle paragrafer og underparagrafer", "Index pages by letter": "Innholdsfortegnelse per bokstav", "search": "s\u00f8k", "Permalink to this definition": "Permalink til denne definisjonen", "Previous topic": "Forrige tittel", "Overview": "Oversikt", "Last updated on %(last_updated)s.": "Sist oppdatert %(last_updated)s.", "Searching": "", "search this documentation": "s\u00f8k i dette dokumentet", "Automatically generated list of changes in version %(version)s": "Automatisk generert liste over endringer i versjon %(version)s", "Full index on one page": "Hele innholdsfortegnelsen p\u00e5 en side", "Enter search terms or a module, class or function name.": "Angi s\u00f8keord eller modul-, klasse- eller funksjonsnavn.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.", "Library changes": "Endringer i biblioteket", "Search Page": "S\u00f8keside", "Search Results": "S\u00f8keresultat"}});