Documentation.addTranslations({"locale": "sv", "plural_expr": "(n != 1)", "messages": {"Next topic": "N\u00e4sta titel", "Index": "Index", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "", "Copyright": "Copyright", "C API changes": "F\u00f6r\u00e4ndringar i C-API", "quick access to all modules": "genv\u00e4g till alla moduler", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "Global Modulindex", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "Index &ndash; %(key)s": "Index &ndash; %(key)s", "General Index": "Huvudindex", "next chapter": "N\u00e4sta kapitel", "Search finished, found %s page(s) matching the search query.": "", "previous chapter": "F\u00f6reg\u00e5ende kapitel", "Permalink to this headline": "Permalink till denna rubrik", "About these documents": "Om dessa dokument", "Preparing search...": "", ", in ": "", "Navigation": "Navigation", "Expand sidebar": "Expandera sidolist", "the documentation for": "", "Complete Table of Contents": "Komplett Inneh\u00e5llsf\u00f6rteckning", "Contents": "Inneh\u00e5ll", "can be huge": "kan bli stort", "Changes in Version %(version)s &mdash; %(docstitle)s": "F\u00f6r\u00e4ndringar i Version %(version)s &mdash; %(docstitle)s", "Other changes": "\u00d6vriga f\u00f6r\u00e4ndringar", "Hide Search Matches": "D\u00f6lj S\u00f6kresultat", "Quick search": "Snabbs\u00f6k", "Show Source": "Visa k\u00e4llfil", "Search": "S\u00f6k", "This Page": "Denna Sida", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "H\u00e4r kan du s\u00f6ka bland dessa dokument. Ange s\u00f6kord nedan och klicka \"s\u00f6k\".\n    S\u00f6kningen m\u00e5ste tr\u00e4ffa p\u00e5 samtliga angivna s\u00f6kord.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Skapad med <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "", "Collapse sidebar": "D\u00f6lj sidolist", "Go": "G\u00e5", "Table Of Contents": "Inneh\u00e5llsf\u00f6rteckning", "Search within %(docstitle)s": "S\u00f6k bland %(docstitle)s", "all functions, classes, terms": "alla funktioner, klasser, villkor", "Please activate JavaScript to enable the search\n    functionality.": "Var god aktivera JavaScript f\u00f6r s\u00f6kfunktionalitet.", "Indices and tables:": "Index och tabeller", "lists all sections and subsections": "lista \u00f6ver alla paragrafer och underparagrafer", "Index pages by letter": "Inneh\u00e5llsf\u00f6rteckning per inledande bokstav", "search": "s\u00f6k", "Permalink to this definition": "Permalink till denna definition", "Previous topic": "F\u00f6reg\u00e5ende titel", "Overview": "\u00d6versikt", "Last updated on %(last_updated)s.": "Senast uppdaterad %(last_updated)s.", "Searching": "", "search this documentation": "s\u00f6k i det h\u00e4r dokumentet", "Automatically generated list of changes in version %(version)s": "Automatiskt genererad lista \u00f6ver f\u00f6r\u00e4ndringar i version %(version)s", "Full index on one page": "Hela inneh\u00e5llsf\u00f6rteckningen p\u00e5 en sida", "Enter search terms or a module, class or function name.": "Ange s\u00f6kord eller modul-, klass- eller funktionsnamn.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.", "Library changes": "F\u00f6r\u00e4ndringar i bibliotek", "Search Page": "S\u00f6ksida", "Search Results": "S\u00f6kresultat"}});