Documentation.addTranslations({"locale": "sl", "plural_expr": "(n%100==1 ? 0 : n%100==2 ? 1 : n%100==3 || n%100==4 ? 2 : 3)", "messages": {"Next topic": "Naslednja tema", "Index": "Abecedni seznam", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "", "Copyright": "Vse pravice pridr\u017eane", "C API changes": "C API spremembe", "quick access to all modules": "hiter dostop do vseh modulov", "&copy; Copyright %(copyright)s.": "&copy; Vse pravice pridr\u017eane %(copyright)s.", "Global Module Index": "Splo\u0161en seznam modulov", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "Index &ndash; %(key)s": "Seznam &ndash; %(key)s", "General Index": "Splo\u0161ni abecedni seznam", "next chapter": "naslednje poglavje", "Search finished, found %s page(s) matching the search query.": "", "previous chapter": "prej\u0161nje poglavje", "Permalink to this headline": "Povezava na naslov", "About these documents": "O dokumentih", "Preparing search...": "", ", in ": "", "Navigation": "Navigacija", "Expand sidebar": "", "the documentation for": "", "Complete Table of Contents": "Popoln Seznam Vsebine", "Contents": "", "can be huge": "lahko je veliko", "Changes in Version %(version)s &mdash; %(docstitle)s": "Spremembe v Verziji %(version)s &mdash; %(docstitle)s", "Other changes": "Ostale spremembe", "Hide Search Matches": "Skrij resultate iskanja", "Quick search": "Hitro iskanje", "Show Source": "Prika\u017ei izvorno kodo", "Search": "I\u0161\u010di", "This Page": "Trenutna stran", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Tukaj lahko i\u0161\u010dete dokumente. Vnesite iskalni\n    niz v polje spodaj in pritisnite \"i\u0161\u010di\". Spro\u017eeno iskanje\n    bo iskalo po vseh besedah v iskalnem nizu. Strani, ki ne\n    vsebujejo vseh besed ne bodo prikazane na seznamu rezultatov.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Narejeno s <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "", "Collapse sidebar": "", "Go": "Potrdi", "Table Of Contents": "Seznam Vsebine", "Search within %(docstitle)s": "I\u0161\u010di med %(docstitle)s", "all functions, classes, terms": "vse funkcije, razredi, izrazi", "Please activate JavaScript to enable the search\n    functionality.": "Za pravilno delovanje Iskanja morete vklopiti\n    JavaScript.", "Indices and tables:": "Kazalo in seznami:", "lists all sections and subsections": "prikazi vse sekcije in podsekcije", "Index pages by letter": "Indeksiraj strani po \u010drki", "search": "i\u0161\u010di", "Permalink to this definition": "Povezava na to definicijo", "Previous topic": "Prej\u0161nja tema", "Overview": "Pregled", "Last updated on %(last_updated)s.": "Zadnji\u010d posodobljeno %(last_updated)s.", "Searching": "", "search this documentation": "i\u0161\u010di po dokumentaciji", "Automatically generated list of changes in version %(version)s": "Avtomatsko generiran seznam sprememb v verziji %(version)s", "Full index on one page": "Poln indeks na eni strani", "Enter search terms or a module, class or function name.": "Vnesi ime modula, razreda ali funkcije.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Vse pravice pridr\u017eane</a> %(copyright)s.", "Library changes": "Spremembe knji\u017enice", "Search Page": "Iskalnik", "Search Results": "Rezultati Iskanja"}});