Documentation.addTranslations({"locale": "cs", "plural_expr": "(n==1) ? 0 : (n>=2 && n<=4) ? 1 : 2", "messages": {"Next topic": "Dal\u0161\u00ed t\u00e9ma", "Index": "Rejst\u0159\u00edk", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "V\u00edtejte! Toto je", "Copyright": "Ve\u0161ker\u00e1 pr\u00e1va vyhrazena", "C API changes": "Zm\u011bny API", "quick access to all modules": "rychl\u00fd p\u0159\u00edstup ke v\u0161em modul\u016fm", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "Celkov\u00fd rejst\u0159\u00edk modul\u016f", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "Vyhled\u00e1v\u00e1n\u00ed nenalezlo \u017e\u00e1dn\u00fd odpov\u00eddaj\u00edc\u00ed dokument. Ujist\u011bte se, \u017ee jste v\u0161echna slova zapsal/a spr\u00e1vn\u011b a \u017ee jste vybral/a dostatek kategori\u00ed.", "Index &ndash; %(key)s": "Rejst\u0159\u00edk &ndash; %(key)s", "General Index": "Obecn\u00fd rejst\u0159\u00edk", "next chapter": "dal\u0161\u00ed kapitola", "Search finished, found %s page(s) matching the search query.": "Vyhled\u00e1v\u00e1n\u00ed dokon\u010deno, str\u00e1nky odpov\u00eddaj\u00edc\u00ed hledan\u00e9mu v\u00fdrazu: %s.", "previous chapter": "p\u0159edchoz\u00ed kapitola", "Permalink to this headline": "Trval\u00fd odkaz na tento nadpis", "About these documents": "O t\u011bchto dokumentech", "Preparing search...": "Vyhled\u00e1v\u00e1n\u00ed se p\u0159ipravuje...", ", in ": ", v ", "Navigation": "Navigace", "Expand sidebar": "Rozbalit bo\u010dn\u00ed li\u0161tu", "the documentation for": "dokumentace pro", "Complete Table of Contents": "Celkov\u00fd obsah", "Contents": "Obsah", "can be huge": "m\u016f\u017ee b\u00fdt obrovsk\u00fd", "Changes in Version %(version)s &mdash; %(docstitle)s": "Zm\u011bny ve verzi %(version)s &mdash; %(docstitle)s", "Other changes": "Ostatn\u00ed zm\u011bny", "Hide Search Matches": "Skr\u00fdt v\u00fdsledky vyhled\u00e1v\u00e1n\u00ed", "Quick search": "Rychl\u00e9 vyhled\u00e1v\u00e1n\u00ed", "Show Source": "Uk\u00e1zat zdroj", "Search": "Vyhled\u00e1v\u00e1n\u00ed", "This Page": "Tato str\u00e1nka", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Toto je vyhled\u00e1vac\u00ed str\u00e1nka. Zadejte kl\u00ed\u010dov\u00e1 slova a klikn\u011bte na \"hledat\". \nVyhled\u00e1v\u00e1n\u00ed automaticky hled\u00e1 v\u0161echna slova, nebudou tedy nalezeny str\u00e1nky obsahuj\u00edc\u00ed jen n\u011bkter\u00e9 z nich.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Vytvo\u0159eno pomoc\u00ed <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "naposledy aktualizov\u00e1no", "Collapse sidebar": "Sbalit bo\u010dn\u00ed li\u0161tu", "Go": "OK", "Table Of Contents": "Obsah", "Search within %(docstitle)s": "Prohledat %(docstitle)s", "all functions, classes, terms": "v\u0161echny funkce, t\u0159\u00eddy, term\u00edny", "Please activate JavaScript to enable the search\n    functionality.": "Pro podporu vyhled\u00e1v\u00e1n\u00ed aktivujte JavaScript.", "Indices and tables:": "Rejst\u0159\u00edky a tabulky:", "lists all sections and subsections": "seznam v\u0161ech sekc\u00ed a podsekc\u00ed", "Index pages by letter": "Rejst\u0159\u00edk podle p\u00edsmene", "search": "hledat", "Permalink to this definition": "Trval\u00fd odkaz na tuto definici", "Previous topic": "P\u0159echoz\u00ed t\u00e9ma", "Overview": "P\u0159ehled", "Last updated on %(last_updated)s.": "Aktualizov\u00e1no dne %(last_updated)s.", "Searching": "Prob\u00edh\u00e1 vyhled\u00e1n\u00ed", "search this documentation": "prohledat tuto dokumentaci", "Automatically generated list of changes in version %(version)s": "Automaticky generovan\u00fd seznam zm\u011bn ve verzi %(version)s", "Full index on one page": "Cel\u00fd rejst\u0159\u00edk na jedn\u00e9 str\u00e1nce", "Enter search terms or a module, class or function name.": "Zadejte hledan\u00e9 term\u00edny nebo jm\u00e9no modulu, t\u0159\u00eddy \u010di funkce.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.", "Library changes": "Zm\u011bny v knihovn\u00e1ch", "Search Page": "Vyhled\u00e1vac\u00ed str\u00e1nka", "Search Results": "V\u00fdsledky vyhled\u00e1v\u00e1n\u00ed"}});