Documentation.addTranslations({"locale": "da", "plural_expr": "(n != 1)", "messages": {"Next topic": "N\u00e6ste emne", "Index": "Indeks", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "Velkommen! Dette er", "Copyright": "Ophavsret", "C API changes": "\u00c6ndringer i C-API", "quick access to all modules": "hurtig adgang til alle moduler", "&copy; Copyright %(copyright)s.": "&copy; Ophavsret %(copyright)s.", "Global Module Index": "Globalt modulindeks", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "Index &ndash; %(key)s": "Indeks &ndash; %(key)s", "General Index": "Generelt indeks", "next chapter": "n\u00e6ste kapitel", "Search finished, found %s page(s) matching the search query.": "", "previous chapter": "forrige kapitel", "Permalink to this headline": "Permalink til denne overskrift", "About these documents": "Om disse dokumenter", "Preparing search...": "Forbereder s\u00f8gning...", ", in ": "", "Navigation": "Navigation", "Expand sidebar": "Udfold sidebj\u00e6lke", "the documentation for": "dokumentationen for", "Complete Table of Contents": "Fuldst\u00e6ndig indholdsfortegnelse", "Contents": "Indhold", "can be huge": "kan v\u00e6re enormt", "Changes in Version %(version)s &mdash; %(docstitle)s": "\u00c6ndringer i version %(version)s &mdash; %(docstitle)s", "Other changes": "Andre \u00e6ndringer", "Hide Search Matches": "Skjul s\u00f8geresultater", "Quick search": "Hurtig s\u00f8gning", "Show Source": "Vis kilde", "Search": "S\u00f8g", "This Page": "Denne side", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Her fra kan du s\u00f8ge i disse dokumenter. Indtast dine s\u00f8geord\n    i boksen nedenfor og klik p\u00e5 \"s\u00f8g\". Bem\u00e6rk at s\u00f8gefunktionen\n    automatisk vil s\u00f8ge p\u00e5 alle ordene. Sider, der indeholder\n    f\u00e6rre ord, vil ikke indg\u00e5 i resultaterne.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Bygget med <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "sidst opdateret", "Collapse sidebar": "Sammenfold sidebj\u00e6lke", "Go": "S\u00f8g", "Table Of Contents": "Indholdsfortegnelse", "Search within %(docstitle)s": "S\u00f8g i %(docstitle)s", "all functions, classes, terms": "alle funktioner, klasser, begreber", "Please activate JavaScript to enable the search\n    functionality.": "Aktiv\u00e9r venligst JavaScript for at aktivere\n    s\u00f8gefunktionalitet.", "Indices and tables:": "Indeks og tabeller:", "lists all sections and subsections": "viser alle afsnit og underafsnit", "Index pages by letter": "Indeks\u00e9r sider efter bogstav", "search": "s\u00f8g", "Permalink to this definition": "Permalink til denne definition", "Previous topic": "Forrige emne", "Overview": "Oversigt", "Last updated on %(last_updated)s.": "Sidst opdateret %(last_updated)s.", "Searching": "S\u00f8ger", "search this documentation": "s\u00f8g i denne dokumentation", "Automatically generated list of changes in version %(version)s": "Automatisk oprettet liste af \u00e6ndringer i version %(version)s", "Full index on one page": "Fuldt indeks p\u00e5 \u00e9n side", "Enter search terms or a module, class or function name.": "Indtast s\u00f8geord eller navnet p\u00e5 et modul, en klasse eller en funktion.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Ophavsret</a> %(copyright)s.", "Library changes": "Biblioteks\u00e6ndringer", "Search Page": "S\u00f8geside", "Search Results": "S\u00f8geresultater"}});