Documentation.addTranslations({"locale": "lv", "plural_expr": "(n%10==1 && n%100!=11 ? 0 : n != 0 ? 1 : 2)", "messages": {"Next topic": "n\u0101ko\u0161a t\u0113ma", "Index": "Indekss", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "", "Copyright": "Copyright", "C API changes": "Izmai\u0146as iek\u0161 C API", "quick access to all modules": "\u0101tra piek\u013cuve visiem moduliem", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "Visp\u0101r\u0113js modu\u013cu indekss", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "Index &ndash; %(key)s": "Indekss &ndash; %(key)s", "General Index": "Visp\u0101r\u0113js indekss", "next chapter": "n\u0101ko\u0161a sada\u013ca", "Search finished, found %s page(s) matching the search query.": "", "previous chapter": "iepriek\u0161\u0113ja sada\u013ca", "Permalink to this headline": "Past\u0101v\u012bga nor\u0101de \u0161o virsrakstu", "About these documents": "Par \u0161iem dokumentiem", "Preparing search...": "", ", in ": "", "Navigation": "Navig\u0101cija", "Expand sidebar": "Izplest s\u0101njoslu", "the documentation for": "", "Complete Table of Contents": "Pilns saturs", "Contents": "Saturs", "can be huge": "var b\u016bt milz\u012bgs", "Changes in Version %(version)s &mdash; %(docstitle)s": "Izmai\u0146as versij\u0101 %(version)s &mdash; %(docstitle)s", "Other changes": "Citas izmai\u0146as", "Hide Search Matches": "Pasl\u0113pt atlases v\u0101rdus", "Quick search": "\u0100tra mekl\u0113\u0161ana", "Show Source": "R\u0101d\u012bt izejas tekstu", "Search": "Mekl\u0113t", "This Page": "\u0160\u012b lappuse", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "\u0160eit J\u016bs varat mekl\u0113t \u0161ajos dokumentos. Nor\u0101diet mekl\u0113jamus v\u0101rdus\n ievada lauka un uzklik\u0161\u0137iniet pogu \"mekl\u0113t\". L\u016bdzu iev\u0113rojiet,\n ka mekl\u0113\u0161anas programma atrad\u012bs tikai tos dokumentus, kuros ir\n visi ievad\u012btie v\u0101rdi. Dokumenti, kuros ir tikai da\u013ca no ievad\u012btiem\n v\u0101rdiem, netiks atlas\u012bti.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Sagatavots izmantojot <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "", "Collapse sidebar": "Sav\u0113rst s\u0101njoslu", "Go": "Izpild\u012bt", "Table Of Contents": "Saturs", "Search within %(docstitle)s": "Mekl\u0113t iek\u0161 %(docstitle)s", "all functions, classes, terms": "visas funkcijas, klases un termini", "Please activate JavaScript to enable the search\n    functionality.": "Lai iesp\u0113jotu mekl\u0113\u0161anu, l\u016bdzu aktiviz\u0113t JavaScript.", "Indices and tables:": "Indeksi un tabulas:", "lists all sections and subsections": "r\u0101da visas sekcijas un apak\u0161sekcijas", "Index pages by letter": "Lappu\u0161u indekss p\u0113c burtiem", "search": "mekl\u0113t", "Permalink to this definition": "Past\u0101v\u012bga nor\u0101de uz \u0161o defin\u012bciju", "Previous topic": "iepriek\u0161\u0113ja t\u0113ma", "Overview": "Apskats", "Last updated on %(last_updated)s.": "P\u0113d\u0113jas izmai\u0146as %(last_updated)s.", "Searching": "", "search this documentation": "mekl\u0113t \u0161aj\u0101 dokument\u0101cij\u0101", "Automatically generated list of changes in version %(version)s": "Autom\u0101tiski sagatavots izmai\u0146u saraksts versijai %(version)s", "Full index on one page": "Pilns indekss vien\u0101 lappus\u0113", "Enter search terms or a module, class or function name.": "Ievadiet mekl\u0113jamus terminus vai modu\u013ca, klases vai funkcijas v\u0101rdu.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.", "Library changes": "Bibliot\u0113kas izmai\u0146as", "Search Page": "Atlases lapa", "Search Results": "Atlases rezult\u0101ti"}});