Documentation.addTranslations({"locale": "eu", "plural_expr": "(n != 1)", "messages": {"Next topic": "Hurrengo gaia", "Index": "Indizea", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "", "Copyright": "Copyright", "C API changes": "C API aldaketak", "quick access to all modules": "modulu guztietara atzipen azkarra", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "Modulu indize globala", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "Index &ndash; %(key)s": "Indizea &ndash; %(key)s", "General Index": "Indize orokorra", "next chapter": "hurrengo kapitulua", "Search finished, found %s page(s) matching the search query.": "", "previous chapter": "aurreko kapitulua", "Permalink to this headline": "Goiburu honetarako esteka iraunkorra", "About these documents": "Dokumentu hauen inguruan", "Preparing search...": "", ", in ": "", "Navigation": "Nabigazioa", "Expand sidebar": "Alboko barra luzatu", "the documentation for": "", "Complete Table of Contents": "Eduki taula osoa", "Contents": "Edukiak", "can be huge": "handia izan daiteke", "Changes in Version %(version)s &mdash; %(docstitle)s": "%(version)s bertsioko aldaketak &mdash; %(docstitle)s", "Other changes": "Beste aldaketak", "Hide Search Matches": "Bilaketa bat-etortzeak ezkutatu", "Quick search": "Bilaketa azkarra", "Show Source": "Iturburua ikusi", "Search": "Bilatu", "This Page": "Orri hau", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Honekin dokumentu hauetan bilatu dezakezu. Sartu zure bilaketa hitzak\nondoko kutxan eta \"bilatu\" sakatu. Kontutan eduki bilaketa funtzioak\nhitz guztiak bilatuko dituela. Hitz gutxiago dituzten orriak ez dira \nemaitzen zerrendan agertuko.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "<a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s erabiliz sortutakoa.", "last updated": "", "Collapse sidebar": "Alboko barra tolestu", "Go": "Joan", "Table Of Contents": "Eduki taula", "Search within %(docstitle)s": "Bilatu %(docstitle)s(e)n", "all functions, classes, terms": "funtzio, klase, termino guztiak", "Please activate JavaScript to enable the search\n    functionality.": "Mesedez, gaitu JavaScript-a bilaketa erabili ahal izateko.", "Indices and tables:": "Indizeak eta taulak:", "lists all sections and subsections": "atal eta azpiatal guztiak zerrendatu", "Index pages by letter": "Indize orriak hizkika", "search": "bilatu", "Permalink to this definition": "Definizio honetarako esteka iraunkorra", "Previous topic": "Aurreko gaia", "Overview": "Gainbegirada", "Last updated on %(last_updated)s.": "Azken aldaketa: %(last_updated)s.", "Searching": "", "search this documentation": "dokumentazio honetan bilatu", "Automatically generated list of changes in version %(version)s": "Automatikoki sortutako %(version)s bertsioaren aldaketen zerrenda", "Full index on one page": "Indize guztia orri batean", "Enter search terms or a module, class or function name.": "Sartu bilaketa terminoa edo modulu, klase edo funtzioaren izena.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.", "Library changes": "Liburutegi aldaketak", "Search Page": "Bilaketa orria", "Search Results": "Bilaketa emaitzak"}});