Documentation.addTranslations({"locale": "fi", "plural_expr": "(n != 1)", "messages": {"Next topic": ">>", "Index": "Sis\u00e4llysluettelo", "%(filename)s &mdash; %(docstitle)s": "", "Welcome! This is": "", "Copyright": "", "C API changes": "", "quick access to all modules": "", "&copy; Copyright %(copyright)s.": "", "Global Module Index": "Yleinen moduulien sis\u00e4llysluettelo", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "Index &ndash; %(key)s": "", "General Index": "Yleinen sis\u00e4llysluettelo", "next chapter": ">>", "Search finished, found %s page(s) matching the search query.": "", "previous chapter": "<<", "Permalink to this headline": "", "About these documents": "Tietoja t\u00e4st\u00e4 documentist\u00e4", "Preparing search...": "", ", in ": "", "Navigation": "Navikointi", "Expand sidebar": "", "the documentation for": "", "Complete Table of Contents": "", "Contents": "", "can be huge": "voi olla iso", "Changes in Version %(version)s &mdash; %(docstitle)s": "Muutos versiosta %(version)s &mdash; %(docstitle)s", "Other changes": "", "Hide Search Matches": "Piilota l\u00f6ydetyt", "Quick search": "Pikahaku", "Show Source": "N\u00e4yt\u00e4 l\u00e4hdekoodina", "Search": "Etsi", "This Page": "T\u00e4m\u00e4 sivu", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Anna hakusanat kokonaan, osasanoilla ei haeta.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "", "last updated": "", "Collapse sidebar": "", "Go": "Siirry", "Table Of Contents": "Sis\u00e4llysluettelo", "Search within %(docstitle)s": "", "all functions, classes, terms": "", "Please activate JavaScript to enable the search\n    functionality.": "Javascript pit\u00e4\u00e4 olla sallittu, jotta etsint\u00e4 toimii.", "Indices and tables:": "", "lists all sections and subsections": "", "Index pages by letter": "Hakemisto aakkostus sivuttain", "search": "etsi", "Permalink to this definition": "", "Previous topic": "<<", "Overview": "Yhteenveto", "Last updated on %(last_updated)s.": "", "Searching": "", "search this documentation": "", "Automatically generated list of changes in version %(version)s": "Automaattisesti luotu muutoshistoria alkaen versiosta %(version)s", "Full index on one page": "Hakemisto yhten\u00e4 luettelona", "Enter search terms or a module, class or function name.": "Anna etsitt\u00e4v\u00e4 termi tai moduuli, luokka tai funktio", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "", "Library changes": "", "Search Page": "Etsi sivu", "Search Results": "Etsinn\u00e4n tulos"}});