Documentation.addTranslations({"locale": "sk", "plural_expr": "(n==1) ? 0 : (n>=2 && n<=4) ? 1 : 2", "messages": {"Next topic": "\u010eal\u0161ia t\u00e9ma", "Index": "Index", "%(filename)s &mdash; %(docstitle)s": "%(filename)s &mdash; %(docstitle)s", "Welcome! This is": "Vitajte! Toto je", "Copyright": "Autorsk\u00e9 pr\u00e1vo", "C API changes": "Zmeny API C", "quick access to all modules": "r\u00fdchly pr\u00edstup ku v\u0161etk\u00fdm modulom", "&copy; Copyright %(copyright)s.": "&copy; Copyright %(copyright)s.", "Global Module Index": "Celkov\u00fd index modulov", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "V\u00e1\u0161mu h\u013eadaniu nezodpoved\u00e1 \u017eiadny dokument. Pros\u00edm, skontrolujte, \u017ee v\u0161etky zadan\u00e9 slov\u00e1 s\u00fa spr\u00e1vne nap\u00edsan\u00e9 a \u017ee ste zvolili vhodn\u00e9 kateg\u00f3rie.", "Index &ndash; %(key)s": "Index &ndash; %(key)s", "General Index": "V\u0161eobecn\u00fd index", "next chapter": "\u010fal\u0161ia kapitola", "Search finished, found %s page(s) matching the search query.": "H\u013eadanie dokon\u010den\u00e9, n\u00e1jden\u00e9 %s strana(y), ktor\u00e9 vyhovuj\u00fa h\u013eadan\u00e9mu v\u00fdrazu.", "previous chapter": "predo\u0161l\u00e1 kapitola", "Permalink to this headline": "Trval\u00fd odkaz na tento nadpis", "About these documents": "O t\u00fdchto dokumentoch", "Preparing search...": "Pr\u00edprava h\u013eadania...", ", in ": ", v ", "Navigation": "Navig\u00e1cia", "Expand sidebar": "Rozbali\u0165 bo\u010dn\u00fd panel", "the documentation for": "dokument\u00e1cia", "Complete Table of Contents": "Celkov\u00fd obsah", "Contents": "Obsah", "can be huge": "m\u00f4\u017ee by\u0165 rozsiahle", "Changes in Version %(version)s &mdash; %(docstitle)s": "Zmeny vo verzii %(version)s &mdash; %(docstitle)s", "Other changes": "Ostatn\u00e9 zmeny", "Hide Search Matches": "Skry\u0165 v\u00fdsledky h\u013eadania", "Quick search": "R\u00fdchle h\u013eadanie", "Show Source": "Zobrazi\u0165 zdroj", "Search": "H\u013eada\u0165", "This Page": "T\u00e1to str\u00e1nka", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "Tu m\u00f4\u017eete h\u013eada\u0165 v tejto dokument\u00e1cii. Zadajte h\u013eadan\u00e9 slov\u00e1\ndo pol\u00ed\u010dka ni\u017e\u0161ie a kliknite na \"h\u013eada\u0165\". Pam\u00e4tajte, \u017ee funkcia\nh\u013eadania bude automaticky h\u013eada\u0165 v\u0161etky slov\u00e1. Strany, ktor\u00e9\nobsahuj\u00fa len niektor\u00e9 zo slov, nebud\u00fa v zozname v\u00fdsledkov.", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "Vytvoren\u00e9 pomocou <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "posledn\u00e1 aktualiz\u00e1cia", "Collapse sidebar": "Zbali\u0165 bo\u010dn\u00fd panel", "Go": "OK", "Table Of Contents": "Obsah", "Search within %(docstitle)s": "H\u013eada\u0165 v %(docstitle)s", "all functions, classes, terms": "v\u0161etky funkcie, triedy, term\u00edny", "Please activate JavaScript to enable the search\n    functionality.": "Pros\u00edm, na zapnutie funkcie h\u013eadania,aktivujte\nJavaScript .", "Indices and tables:": "Indexy a tabu\u013eky", "lists all sections and subsections": "zoznam sekci\u00ed a podsekci\u00ed", "Index pages by letter": "Indexov\u00e9 str\u00e1nky po p\u00edsmen\u00e1ch", "search": "h\u013eada\u0165", "Permalink to this definition": "Trval\u00fd odkaz na t\u00fato defin\u00edciu", "Previous topic": "Predo\u0161l\u00e1 t\u00e9ma", "Overview": "Preh\u013ead", "Last updated on %(last_updated)s.": "Naposledy aktualizovan\u00e9 %(last_updated)s.", "Searching": "H\u013eadanie", "search this documentation": "h\u013eada\u0165 v tejto dokument\u00e1cii", "Automatically generated list of changes in version %(version)s": "Automaticky generovan\u00fd zoznam zmien vo verzii %(version)s", "Full index on one page": "Cel\u00fd index na jednej strane", "Enter search terms or a module, class or function name.": "Zadajte h\u013eadan\u00e9 v\u00fdrazy alebo modul, triedu, \u010di meno funkcie.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.", "Library changes": "Zmeny kni\u017enice", "Search Page": "Str\u00e1nka h\u013eadania", "Search Results": "V\u00fdsledky h\u013eadania"}});