Documentation.addTranslations({"locale": "vi", "plural_expr": "0", "messages": {"Next topic": "Ch\u1ee7 \u0111\u1ec1 ti\u1ebfp", "Index": "", "%(filename)s &mdash; %(docstitle)s": "", "Welcome! This is": "Ch\u00e0o m\u1eebng! \u0110\u00e2y l\u00e0", "Copyright": "B\u1ea3n quy\u1ec1n", "C API changes": "", "quick access to all modules": "truy c\u1eadp nhanh t\u1ea5t c\u1ea3 c\u00e1c m\u00f4-\u0111un", "&copy; Copyright %(copyright)s.": "&copy; B\u1ea3n quy\u1ec1n thu\u1ed9c %(copyright)s.", "Global Module Index": "Ch\u1ec9 M\u1ee5c M\u00f4-\u0111un To\u00e0n C\u1ee5c", "Your search did not match any documents. Please make sure that all words are spelled correctly and that you've selected enough categories.": "", "Index &ndash; %(key)s": "Ch\u1ec9 m\u1ee5c &ndash; %(key)s", "General Index": "Ch\u1ec9 m\u1ee5c chung", "next chapter": "ch\u01b0\u01a1ng ti\u1ebfp", "Search finished, found %s page(s) matching the search query.": "", "previous chapter": "ch\u01b0\u01a1ng tr\u01b0\u1edbc ", "Permalink to this headline": "", "About these documents": "V\u1ec1 c\u00e1c t\u00e0i li\u1ec7u n\u00e0y", "Preparing search...": "", ", in ": "", "Navigation": "\u0110i\u1ec1u h\u01b0\u1edbng", "Expand sidebar": "", "the documentation for": "t\u00e0i li\u1ec7u cho", "Complete Table of Contents": "M\u1ee5c L\u1ee5c \u0110\u1ea7y \u0110\u1ee7", "Contents": "", "can be huge": "c\u00f3 th\u1ec3 r\u1ea5t nhi\u1ec1u", "Changes in Version %(version)s &mdash; %(docstitle)s": "", "Other changes": "", "Hide Search Matches": "", "Quick search": "", "Show Source": "Hi\u1ec3n th\u1ecb m\u00e3 ngu\u1ed3n", "Search": "T\u00ecm Ki\u1ebfm", "This Page": "", "From here you can search these documents. Enter your search\n    words into the box below and click \"search\". Note that the search\n    function will automatically search for all of the words. Pages\n    containing fewer words won't appear in the result list.": "", "Created using <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.": "\u0110\u01b0\u1ee3c t\u1ea1o nh\u1edd <a href=\"http://sphinx-doc.org/\">Sphinx</a> %(sphinx_version)s.", "last updated": "c\u1eadp nh\u1eadt m\u1edbi nh\u1ea5t", "Collapse sidebar": "", "Go": "Th\u1ef1c hi\u1ec7n", "Table Of Contents": "M\u1ee5c L\u1ee5c", "Search within %(docstitle)s": "T\u00ecm ki\u1ebfm trong %(docstitle)s", "all functions, classes, terms": "t\u1ea5t c\u1ea3 c\u00e1c h\u00e0m, l\u1edbp, thu\u1eadt ng\u1eef", "Please activate JavaScript to enable the search\n    functionality.": "H\u00e3y b\u1eadt JavaScript \u0111\u1ec3 d\u00f9ng t\u00ednh n\u0103ng\nt\u00ecm ki\u1ebfm.", "Indices and tables:": "C\u00e1c ch\u1ec9 m\u1ee5c v\u00e0 b\u1ea3ng bi\u1ec3u:", "lists all sections and subsections": "li\u1ec7t k\u00ea t\u1ea5t c\u1ea3 c\u00e1c m\u1ee5c v\u00e0 m\u1ee5c con", "Index pages by letter": "C\u00e1c trang ch\u1ec9 m\u1ee5c theo ch\u1eef c\u00e1i", "search": "", "Permalink to this definition": "", "Previous topic": "Ch\u1ee7 \u0111\u1ec1 tr\u01b0\u1edbc", "Overview": "T\u1ed5ng quan", "Last updated on %(last_updated)s.": "C\u1eadp nh\u1eadt m\u1edbi nh\u1ea5t v\u00e0o %(last_updated)s.", "Searching": "", "search this documentation": "t\u00ecm ki\u1ebfm trong t\u00e0i li\u1ec7u n\u00e0y", "Automatically generated list of changes in version %(version)s": "", "Full index on one page": "To\u00e0n b\u1ed9 ch\u1ec9 m\u1ee5c tr\u00ean m\u1ed9t trang", "Enter search terms or a module, class or function name.": "Nh\u1eadp thu\u1eadt ng\u1eef, t\u00ean m\u1ed9t m\u00f4-\u0111un, l\u1edbp hay h\u00e0m c\u1ea7n t\u00ecm.", "&copy; <a href=\"%(path)s\">Copyright</a> %(copyright)s.": "&copy; <a href=\"%(path)s\">B\u1ea3n quy\u1ec1n thu\u1ed9c</a> %(copyright)s.", "Library changes": "", "Search Page": "", "Search Results": ""}});